package org.objectweb.fractal.bf.connectors.ws;

import java.util.Map;
import java.util.logging.Logger;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.bf.AbstractBindingFactoryPlugin;
import org.objectweb.fractal.bf.BindHints;
import org.objectweb.fractal.bf.BindingFactoryException;
import org.objectweb.fractal.bf.ExportHints;
import org.objectweb.fractal.bf.SkeletonGenerationException;
import org.objectweb.fractal.bf.proxies.StubComponentFactory;
import org.objectweb.fractal.util.Fractal;

/**
 * A WS-Connector can export a Fractal interface to make it available via SOAP.
 * 
 * @author Valerio Schiavoni <valerio.schiavoni@gmail.com>
 * 
 */
public class WsConnector extends
		AbstractBindingFactoryPlugin<WsExportHints, WsBindHints> {

	Logger log = Logger.getLogger(WsConnector.class.getCanonicalName());

	/**
	 * Default 0-args constructor to please Julia.
	 */
	public WsConnector() {
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#finalizeSkeleton(org.objectweb.fractal.api.Component,
	 *      java.util.Map)
	 */
	public void finalizeSkeleton(Component skel,
			Map<String, String> finalizationHints)
			throws BindingFactoryException {

		log.info("Finalizing ws-generated skeleton..");

		try {
			WsServiceAttributes wsServiceAttributes = (WsServiceAttributes) skel
					.getFcInterface("serviceAttributes");

			wsServiceAttributes
					.finalizeConfig(finalizationHints.get("itfName"));

			wsServiceAttributes.startWs();

		} catch (NoSuchInterfaceException e) {
			throw new BindingFactoryException(
					"Cannot find 'serviceAttributes' fcInterface on the specifies skeleton component",
					e);
		}

	}

	@Override
	public String toString() {
		return "SOAP/CXF Connector Plugin";
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#getStubComponentFactory()
	 */
	public StubComponentFactory getStubComponentFactory() {
		return new WsConnectorStubComponentFactoryImpl();
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#createSkel(java.lang.String,
	 *      java.lang.Object, org.objectweb.fractal.api.Component,
	 *      org.objectweb.fractal.bf.ExportHints)
	 */
	public Component createSkel(String interfaceName, Object serverItf,
			Component owner, WsExportHints wsExportHints)
			throws SkeletonGenerationException {
		log.info("WsConnector - Creating skeleton..");

		String address = wsExportHints.getAddress();

		if (address == null) {
			address = WsConnectorConstants.DEFAULT_ADDRESS;
		}

		Component skeleton = null;
		try {
			Factory adlFactory = FactoryFactory
					.getFactory(FactoryFactory.FRACTAL_BACKEND);

			skeleton = (Component) adlFactory.newComponent(
					"org.objectweb.fractal.bf.connectors.ws.WsServerBridge",
					null);

			// start the skeleton
			Fractal.getLifeCycleController(skeleton).startFc();

			WsServiceAttributes wsServiceAttributes = (WsServiceAttributes) skeleton
					.getFcInterface("serviceAttributes");

			InterfaceType it = (InterfaceType) ((Interface) serverItf)
					.getFcItfType();
			Class<?> serviceClass = Thread.currentThread()
					.getContextClassLoader().loadClass(it.getFcItfSignature());
			log.finest("Service class: " + serviceClass
					+ " correctly recovered from context class loader");

			wsServiceAttributes.configureWsWith(address, serviceClass);

		} catch (ADLException e) {
			throw new SkeletonGenerationException(
					"Some problem occured while  loading the ADL definition of the skeleton",
					e);
		} catch (NoSuchInterfaceException e) {
			throw new SkeletonGenerationException(
					"Some problem occured while loading the skeleton ", e);
		} catch (IllegalLifeCycleException e) {
			throw new SkeletonGenerationException(
					"Cannot start the skeleton component", e);
		} catch (ClassNotFoundException e) {
			throw new SkeletonGenerationException(
					"Cannot find class for the service", e);
		}

		return skeleton;
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#getExportHints()
	 */
	public ExportHints getExportHints() {
		WsExportHints wsExportHints = new WsExportHints();
		log.fine("Initial hints:" + this.initialHints);
		wsExportHints.setAddress(this.initialHints
				.get(WsConnectorConstants.ADDRESS));

		return wsExportHints;
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#getPluginIdentifier()
	 */
	public String getPluginIdentifier() {
		return "ws";
	}

	/**
	 * @see org.objectweb.fractal.bf.BindingFactoryPlugin#getBindHints()
	 */
	public BindHints getBindHints() {
		WsBindHints wsBindHints = new WsBindHints();
		wsBindHints.setAddress(this.initialHints
				.get(WsConnectorConstants.ADDRESS));
		log.fine("WsBindHints: " + wsBindHints);
		return wsBindHints;
	}

}
